#ifndef GLOBALS_HH
#define GLOBALS_HH

// stuff used by more than one file
// mostly symbol conversion, deque hashing etc.

#include <deque>
#include <string>
#include <ext/hash_map>
#include <iterator>

using namespace std;
using namespace __gnu_cxx;

// we treat everything as a string, also the 1-character symbols
typedef string Symbol;
typedef unsigned int SymbolID;

Symbol char2symbol(char c);

class symbol_hash 
{
  hash<const char *> h;
public:
  size_t operator()(Symbol const &s) const
  {
    return h(s.c_str());
  }
};

// conversion between Symbols (words, strings) and ids
extern hash_map<Symbol, unsigned int, symbol_hash> symbolid;
extern vector<Symbol> id2symbol;

SymbolID getSymbolID(Symbol const &s);

class deque_hash
{
  static const unsigned int numprimes;
  static const unsigned int primes[];

public:
  size_t operator()(deque<SymbolID> const &dq) const
  {
    // this only works well if primes[] is long enough!
    // also do not use if the size of the arguments varies!
    size_t result = 0;

    for (unsigned int i = 0; i < dq.size(); i++)
      result ^= dq[i] * primes[i%numprimes];

    return result;
  }
};

ostream &operator<<(ostream & os, deque<SymbolID> const &dq);
void print_deque_readable(deque<SymbolID> const &dq, ostream &os);

// error codes
const int error_code_usage = 1;
const int error_code_max_order = 2;
const int error_code_stream_too_short = 3;

#endif
